/*
 * 
 * Copyright (c) CSIRO Astronomy and Space Science
 * Author Arkadi.Kosmynin@csiro.au
 * 
 */

package au.csiro.cass.arch.auth;

import java.io.IOException;

import au.csiro.cass.arch.utils.URLSplit;

public class Permissions
{
  String        file ; // url these permissions apply to
  String        type ; // type of the url: "f" - file, "d" - directory
  // note that /b/c/d refer to directory d (if d is a directory)
  //       but /b/c/d/ refer to default file in directory d
  URLSplit     split ; // split into path and file name
  String       userr ; // list of users having read access
  String       userw ; // list of users having read and write access
  String      groupr ; // list of groups having read access
  String      groupw ; // list of groups having read and write access
  String      owners ; // list of users having admin access
  String  accessType ; // "s" - standard, "i" - inherited
  // Everything is taken from parent node when access is inherited, and this is the
  // default access type. The only reason to have an "i" Permissions object is to
  // change access from "s" where it was set to "s" before.
  
  /**
   * Parameterless constructor
   */  
  public Permissions() {}
  
  /**
   * Parses configuration permissions record and creates a Permissions object
   * @param record - permissions record as described in sample configuration file
   * @return Permissions object
   */
  static public Permissions newPermissions( String record ) throws IOException
  {
   // permissions record example:
   // http://a.b.c/d/e | userr1 userr2 | userw1 | groupr1 | groupw1 | amdin | s  
   Permissions p = new Permissions() ;
   String[] fields = record.split( "\\|" ) ;
   p.type       = fields[ 0 ].trim() ; if ( p.type.length() == 0 ) p.type = "f" ;   
   p.file       = fields[ 1 ].trim() ; if ( p.file.length() == 0 ) p.file = null ;   
   p.groupr     = fields[ 2 ].trim() ; if ( p.groupr.length() == 0 ) p.groupr = null ; 
   p.groupw     = fields[ 3 ].trim() ; if ( p.groupw.length() == 0 ) p.groupw = null ; 
   p.userr      = fields[ 4 ].trim() ; if ( p.userr.length() == 0 ) p.userr = null ; 
   p.userw      = fields[ 5 ].trim() ; if ( p.userw.length() == 0 ) p.userw = null ; 
   p.owners     = fields[ 6 ].trim() ; if ( p.owners.length() == 0 ) p.owners = null ; 
   p.accessType = fields[ 7 ].trim() ; if ( p.accessType.length() == 0 ) p.accessType = null ;
   p.split = URLSplit.newURLSplit( p.file, null, p.type ) ;
   return p ;
  }

/**
 * @return the url
 */
public String getFile() {
	return file;
}

/**
 * @param url the url to set
 */
public void setFile( String file, String type ) throws Exception 
{
	this.file = file;
	this.type = type ;
	this.split = URLSplit.newURLSplit( this.file, null, type ) ;
}

/**
 * @return file name
 */
public String getName()
{
  return split.getName() ;
}

/**
 * @return file path
 */
public String getPath()
{
  return split.getPath() ;
}

/**
 * @return the userr
 */
public String getUserr() {
	return userr;
}

/**
 * @return array of users having read access 
 */
public String[] getUserrArray()
{
  if ( userr != null )return userr.split( " " ) ;
  else return null ;
}

/**
 * @param userr the userr to set
 */
public void setUserr(String userr) {
	this.userr = userr;
}

/**
 * @return the userw
 */
public String getUserw() {
	return userw;
}

/**
 * @return array of users having read and write access 
 */
public String[] getUserwArray()
{
  if ( userw != null )return userw.split( " " ) ;
  else return null ;
}



/**
 * @param userw the userw to set
 */
public void setUserw(String userw) {
	this.userw = userw;
}

/**
 * @return the groupr
 */
public String getGroupr() {
	return groupr;
}

/**
 * @return array of groups having read access 
 */
public String[] getGrouprArray()
{
  if ( groupr != null )return groupr.split( " " ) ;
  else return null ;
}

/**
 * @param groupsr the groupsr to set
 */
public void setGroupr( String groupr ) {
	this.groupr = groupr;
}

/**
 * @return the groupsw
 */
public String getGroupw() {
	return groupw;
}

/**
 * @return array of groups having write access 
 */
public String[] getGroupwArray()
{
  if ( groupw != null )return groupw.split( " " ) ;
  else return null ;
}

/**
 * @param groupsw the groupsw to set
 */
public void setGroupw(String groupw) {
	this.groupw = groupw;
}

/**
 * @return the owners
 */
public String getOwners() {
	return owners;
}

/**
 * @return array of owners
 */
public String[] getOwnersArray()
{
  if ( owners != null ) return owners.split( " " ) ;
  else return null ;
}

/**
 * @param owners the owners to set
 */
public void setOwners(String owners) {
	this.owners = owners;
}

/**
 * @return the accessType
 */
public String getAccessType() {
	return accessType;
}

/**
 * @param accessType the accessType to set
 */
public void setAccessType(String accessType) {
	this.accessType = accessType;
}

}
