/*
 * 
 * Copyright (c) CSIRO Astronomy and Space Science
 * Author Arkadi.Kosmynin@csiro.au
 * 
 */

package au.csiro.cass.arch.auth;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

public class SearchProfile
{
	  String        id ; // id of the client
	  String  password ; // client's password
	  String     sites ; // list of sites allowed to be searched
	  String     areas ; // list of areas allowed to be searched
	  String    groups ; // list of groups allowed to be presented
	  String     users ; // list of users allowed to be presented
	  boolean    guest ; // true if this is guest profile

	  HashSet<String>    siteS ; 
	  HashSet<String>    areaS ; 
	  HashSet<String>   groupS ; 
	  HashSet<String>    userS ;
	
	  
   /**
	* Parses configuration search profile (front-end profile) record and creates a SearchProfile object
	* @param record - search profile record as described in sample configuration file
	* @return SearchProfile object
	*/  
	static public SearchProfile newSearchProfile( String record ) throws IOException
	{
	 // search profile record example:
	 // myId | myPassword | site1 site2 | area1 area2 | group1 group2 | user1 user2  
	 SearchProfile c = new SearchProfile() ;
	 c.guest = false ;  
	 String[] fields = record.split( "\\|" ) ;
	 if ( fields.length < 2 )
		   throw new IOException( "Id and password are required parameters in search profile." ) ;
	 c.id       = fields[ 0 ].trim() ;
	 c.password = fields[ 1 ].trim() ; // id and password are the only requred parameters
	 if ( fields.length > 2 )
	 {
	  c.sites = fields[ 2 ].trim() ; if ( c.sites.length() == 0 ) c.sites = null ;
	                                        else c.siteS = newSet( c.sites ) ;
	  if ( fields.length > 3 )
	  {    	
	   c.areas = fields[ 3 ].trim() ; if ( c.areas.length() == 0 ) c.areas = null ; 
                                                    else c.areaS = newSet( c.areas ) ;
	   if ( fields.length > 4 )
	   {
	    c.groups = fields[ 4 ].trim() ; if ( c.groups.length() == 0 ) c.groups = null ; 
                                                   else c.groupS = newSet( c.groups ) ;
	    if ( fields.length > 5 )
	    {
	     c.users = fields[ 5 ].trim() ; if ( c.users.length() == 0 ) c.users = null ; 
                                                     else c.userS = newSet( c.users ) ;
	    }
	   }
	  }
	 }
	 return c ;
	}
	  
	static public SearchProfile newGuestSearchProfile()
    {
	 SearchProfile c = new SearchProfile() ;
	 c.guest = true ;
	 return c ;
	}
	  
	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @return the sites
	 */
	public String getSites() {
		return sites;
	}

	/**
	 * @return the areas
	 */
	public String getAreas() {
		return areas;
	}

	/**
	 * @return the groups
	 */
	public String getGroups() {
		return groups;
	}

	/**
	 * @return the users
	 */
	public String getUsers() {
		return users;
	}

	/**
	 * @return the siteS
	 */
	public HashSet<String> getSiteS() {
		return siteS;
	}

	/**
	 * @return the areaS
	 */
	public HashSet<String> getAreaS() {
		return areaS;
	}

	/**
	 * @return the groupS
	 */
	public HashSet<String> getGroupS() {
		return groupS;
	}

	/**
	 * @return the userS
	 */
	public HashSet<String> getUserS() {
		return userS;
	}

	SearchProfile() {}
	  
	static public HashSet<String> newSet( String str )
	{ 
		String[] a = str.split( " " ) ;
		HashSet<String> set = new HashSet<String>( a.length ) ;
		for ( int i = 0 ; i < a.length ; i++ )
		{
		  String aa = a[ i ].trim() ;
		  if ( aa.length() == 0 ) continue ;
		  set.add( aa )	;
		}
		return set ;    
	  }

	public String getSites( String sites )
	  {
	   if ( guest ) return sites ;
       if ( sites == null || sites.length() == 0 ) return this.sites ;
       else return intersect( siteS, sites ) ;
	  }

	public String getAreas( String areas )
	  {
	   if ( guest ) return areas ;
       if ( areas == null || areas.length() == 0 ) return this.areas ;
       else return intersect( areaS, areas ) ;
	  }

	public String getGroups( String groups )
	  {
	   if ( guest ) return( "public" ) ;
       if ( groups == null || groups.length() == 0 ) return this.groups ;
       else return intersect( groupS, groups ) ;
	  }

	public String getUsers( String users )
	  {
	   if ( guest ) return ( "guest" ) ;
       if ( users == null || users.length() == 0 ) return this.users ;
       else return intersect( userS, users ) ;
	  }
	  
	  String intersect( Set<String> set, String str )
	  {
	   if ( set == null ) return str ; // no limitations
	   String[] a = str.split( " " ) ;
	   StringBuffer buf = new StringBuffer() ;
	   
	   for ( int i = 0 ; i < a.length ; i++ )
	   {
		 String aa = a[ i ].trim() ;
		 if ( set.contains( aa ) )
			{ if ( buf.length() > 0 ) buf.append( " " ) ; buf.append( aa ) ; } 		   
	   }
	   
	   return buf.toString() ;
	  }
}
