/*
 * 
 * Copyright (c) CSIRO Astronomy and Space Science
 * Author Arkadi.Kosmynin@csiro.au
 * 
 */
package au.csiro.cass.arch.index;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A "special" area, containing urls found by log processing, but not by the crawler
 * 
 * @author Arkadi Kosmynin
 *
 */
public class LogLinks extends IndexArea
{
 public static final Logger LOG = LoggerFactory.getLogger( LogLinks.class ) ;
 
 /**
  *  Default parameterless constructor, does nothing
  *
  */
 public LogLinks() {}
 
 
/**
 * LogLinks factory
 * 
 * @param site parent site object
 * @return LogLinks object, a special case of IndexArea
 * @throws Exception
 *
 */
 static public LogLinks newLogLinks( IndexSite site ) throws Exception
 {
   LogLinks area = new LogLinks() ;
   
   LOG.info( "New index area: loglinks " ) ;
   area.name = "loglinks" ;
   area.enabled = site.cfg.get( "enabled.loglinks", true ) ; 
   area.site = site ;
   area.interval = site.cfg.get( "interval.loglinks", 1 ) ; 
   area.areaClause = " where site=\'" + site.name + "\' and area=\'loglinks\'";
   area.exclusions = site.cfg.getAll( "exclude.loglinks", true ) ;
   area.inclusions = site.cfg.getAll( "include.loglinks", true ) ;
   area.setCrawlingDays() ;           
   area.enabled = site.cfg.get( "enabled." + area.name, true ) ; 
   area.site.indexer.db.readDbValues( area ) ;
   area.depth = site.cfg.getInherited( "depth.loglinks", 1 ) ; 

   return area ;
 }

 // Only reindex loglinks when time comes
 boolean changed() { return false ; }
 
 /**
  *  Overrides the IndexArea injector. Takes URLs to inject from the database.
  *  One URL per line.
  * 
  */ 
 boolean inject() throws Exception
 {
  int lines = site.db.readUnindexedURLs( this ) ;

  if ( lines == 0 ) return false ;
  return true ;
 }

}
