/*
 * 
 * Copyright (c) CSIRO Astronomy and Space Science
 * Author Arkadi.Kosmynin@csiro.au
 * 
 */

package au.csiro.cass.arch.index;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Matcher of URL -> site and area
 * 
 */
public class SiteMatcher
{
  Map<String, AreaMatcher> areaMatchers ; // site host address -> area matcher
  
  public SiteMatcher()
  {
    areaMatchers = new HashMap<String, AreaMatcher>() ;	  
  }
  
  
  /** 
   * SiteMatcher factory. Creates an site matcher based on a map of roots, inclusions and exclusions.
   * The keys have form of <site name>.<area name>. 
   * @param roots - site name
   * @return created matcher
   */
  public static SiteMatcher newSiteMatcher( Map<String, ArrayList<String>[]> roots ) throws Exception
  {
	SiteMatcher matcher = new SiteMatcher() ;
	Iterator it = roots.entrySet().iterator() ;  
    while( it.hasNext() )
	 {
       Map.Entry<String, ArrayList<String>[]> ent = (Map.Entry<String, ArrayList<String>[]>)it.next() ;
       String[] kk = ent.getKey().split( "\n" ) ;
       if ( kk[ 1 ].equals( "bookmarks" ) ) continue ;
       ArrayList<String>[] paths = ent.getValue() ;
       String u = null ;
       if ( paths[ 2 ] != null && paths[ 2 ].size() > 0 ) u = paths[ 2 ].get( 0 ) ;
       else if ( paths[ 1 ] != null && paths[ 1 ].size() > 0 ) u = paths[ 1 ].get( 0 ) ;
       else if ( paths[ 0 ] != null && paths[ 0 ].size() > 0 ) u = paths[ 0 ].get( 0 ) ;
       if ( u == null ) continue ;
       URL url = new URL( u ) ; 
       String host = url.getHost().toLowerCase() ;
       AreaMatcher areaMatcher = matcher.areaMatchers.get( host ) ;
       if ( areaMatcher == null )
         {
    	   areaMatcher = AreaMatcher.newAreaMatcher( kk[ 0 ] ) ;
           matcher.areaMatchers.put( host, areaMatcher ) ;
         }    	   
       areaMatcher.addArea( kk[ 1 ], paths[ 2 ], paths[ 0 ], paths[ 1 ] ) ;
	 }  
    return matcher ;
  }

  
  /** 
   * Identify site and area based on the URL.
   * @param url - URL to use for identification.
   * @return SiteArea container with site and area names.
   */
  public SiteArea getSiteArea( String url ) throws Exception
  {
	String u = url.toLowerCase() ;
	URL uu = new URL( u ) ;
	AreaMatcher matcher = areaMatchers.get( uu.getHost() ) ;
	if ( matcher == null ) return null ;
	String area = matcher.getArea( u ) ;
	return new SiteArea( matcher.site, area ) ;
  }

}
